/*********************************************************************
  This is an example for our nRF51822 based Bluefruit LE modules

  Pick one up today in the adafruit shop!

  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  MIT license, check LICENSE for more information
  All text above, and the splash screen below must be included in
  any redistribution
*********************************************************************/

#include <string.h>
#include <Arduino.h>
#include <SPI.h>
#if not defined (_VARIANT_ARDUINO_DUE_X_) && not defined (_VARIANT_ARDUINO_ZERO_)
#include <SoftwareSerial.h>
#endif

#include "Adafruit_BLE.h"
#include "Adafruit_BluefruitLE_SPI.h"
#include "Adafruit_BluefruitLE_UART.h"

#include "BluefruitConfig.h"

#include <Adafruit_NeoPixel.h>

/*=========================================================================
    APPLICATION SETTINGS

      FACTORYRESET_ENABLE       Perform a factory reset when running this sketch
     
                                Enabling this will put your Bluefruit LE module
                              in a 'known good' state and clear any config
                              data set in previous sketches or projects, so
                                running this at least once is a good idea.
     
                                When deploying your project, however, you will
                              want to disable factory reset by setting this
                              value to 0.  If you are making changes to your
                                Bluefruit LE device via AT commands, and those
                              changes aren't persisting across resets, this
                              is the reason why.  Factory reset will erase
                              the non-volatile memory where config data is
                              stored, setting it back to factory default
                              values.
         
                                Some sketches that require you to bond to a
                              central device (HID mouse, keyboard, etc.)
                              won't work at all with this feature enabled
                              since the factory reset will clear all of the
                              bonding data stored on the chip, meaning the
                              central device won't be able to reconnect.
    PIN                       Which pin on the Arduino is connected to the NeoPixels?
    NUMPIXELS                 How many NeoPixels are attached to the Arduino?
    -----------------------------------------------------------------------*/
#define FACTORYRESET_ENABLE     1

#define PIN                    9
#define NUMPIXELS               24
/*=========================================================================*/

Adafruit_NeoPixel pixel = Adafruit_NeoPixel(NUMPIXELS, PIN, NEO_GRBW);

Adafruit_BluefruitLE_SPI ble(BLUEFRUIT_SPI_CS, BLUEFRUIT_SPI_IRQ, BLUEFRUIT_SPI_RST);


// A small helper
void error(const __FlashStringHelper*err) {
  Serial.println(err);
  while (1);
}

// function prototypes over in packetparser.cpp
uint8_t readPacket(Adafruit_BLE *ble, uint16_t timeout);
float parsefloat(uint8_t *buffer);
void printHex(const uint8_t * data, const uint32_t numBytes);

// the packet buffer
extern uint8_t packetbuffer[];


/**************************************************************************/
/*!
    @brief  Sets up the HW an the BLE module (this function is called
            automatically on startup)
*/
/**************************************************************************/
//additional variables

//Color
uint8_t red = 255;
uint8_t green = 255;
uint8_t blue = 255;
uint8_t white = 255;
uint8_t animationState = 1;

uint8_t myColors[][3] = {
  {232, 100, 255},   // pink
  {238, 238, 36},   // yellow
  {54, 204, 204},   // teal
  {36, 40, 233},    // Dark blue
  {76, 183, 98},   // Lt Green
};

#define FAVCOLORS sizeof(myColors) / 3

int pos = 0, dir = 1; // Position, direction of "eye" for larson scanner animation

int ledFadeTime = 10;
int totalLEDs = 24;


void setup(void)
{
  //while (!Serial);  // required for Flora & Micro
  delay(500);

  // turn off neopixel
  pixel.begin(); // This initializes the NeoPixel library.

  for (uint8_t i = 0; i < NUMPIXELS; i++) {
    pixel.setPixelColor(i, pixel.Color(0, 0, 0)); // off
  }
  colorWipe(pixel.Color(255, 255, 255), 15);
  colorWipe(pixel.Color(0, 0, 0), 15);
  pixel.show();

  Serial.begin(115200);
  Serial.println(F("Adafruit Bluefruit Neopixel Color Picker Example"));
  Serial.println(F("------------------------------------------------"));

  /* Initialise the module */
  Serial.print(F("Initialising the Bluefruit LE module: "));

  if ( !ble.begin(VERBOSE_MODE) )
  {
    error(F("Couldn't find Bluefruit, make sure it's in CoMmanD mode & check wiring?"));
  }
  Serial.println( F("OK!") );

  if ( FACTORYRESET_ENABLE )
  {
    /* Perform a factory reset to make sure everything is in a known state */
    Serial.println(F("Performing a factory reset: "));
    if ( ! ble.factoryReset() ) {
      error(F("Couldn't factory reset"));
    }
  }

  /* Disable command echo from Bluefruit */
  ble.echo(false);

  Serial.println("Requesting Bluefruit info:");
  /* Print Bluefruit information */
  ble.info();

  Serial.println(F("Please use Adafruit Bluefruit LE app to connect in Controller mode"));
  Serial.println(F("Then activate/use the sensors, color picker, game controller, etc!"));
  Serial.println();

  ble.verbose(false);  // debug info is a little annoying after this point!

  /* Wait for connection */
  while (! ble.isConnected()) {
    delay(500);
  }

  Serial.println(F("***********************"));

  // Set Bluefruit to DATA mode
  Serial.println( F("Switching to DATA mode!") );
  ble.setMode(BLUEFRUIT_MODE_DATA);

  Serial.println(F("***********************"));
}

/**************************************************************************/
/*!
    @brief  Constantly poll for new command or response data
*/
/**************************************************************************/
void loop(void) {
  /* Wait for new data to arrive */
  uint8_t len = readPacket(&ble, BLE_READPACKET_TIMEOUT);
  if (len == 0) return;

  // Color
  if (packetbuffer[1] == 'C') {
    uint8_t red = packetbuffer[2];
    uint8_t green = packetbuffer[3];
    uint8_t blue = packetbuffer[4];
    Serial.print ("RGB #");
    if (red < 0x10) Serial.print("0");
    Serial.print(red, HEX);
    if (green < 0x10) Serial.print("0");
    Serial.print(green, HEX);
    if (blue < 0x10) Serial.print("0");
    Serial.println(blue, HEX);

    for (uint8_t i = 0; i < NUMPIXELS; i++) {
      pixel.setPixelColor(i, pixel.Color(red, green, blue, white));
    }
    pixel.show(); // turns pixels off
  }

  // Buttons
  if (packetbuffer[1] == 'B') {

    uint8_t buttnum = packetbuffer[2] - '0';
    boolean pressed = packetbuffer[3] - '0';
    Serial.print ("Button "); Serial.print(buttnum);
    animationState = buttnum;
    if (pressed) {
      Serial.println(" pressed");
    } else {
      Serial.println(" released");
    }


    // STATE1 ////////////////////////////////////////////////////////////////////////////////////////////////////

    if (animationState == 1) {

      rgbFadeInAndOutPink(2);
      colorWipe(pixel.Color(0, 0, 0), 2);
   
      delay(2000);
       rgbFadeInAndOutTeal(2);
    
       delay(2000);
      rgbFadeInAndOutRed(2);

       delay(2000);
        rgbFadeInAndOut4(2);
        colorWipe(pixel.Color(0, 0, 0), 2);
   
       delay(2000);
       flashRandom(20, 5);  
      flashRandom(20, 10);

       delay(2000);
       flashRandom(20, 5);  
      flashRandom(20, 10);

      delay(2000);
      flashRandom(20, 5);  
      flashRandom(20, 10);
      
      delay(2000);
      pixel.show();

    }

    // STATE2 ////////////////////////////////////////////////////////////////////////////////////////////////////

    if (animationState == 2) {
      pixel.setBrightness(255);
      larsonScanner1(25);
      larsonScanner2(25);
colorWipe(pixel.Color(0, 0, 0), 20);
         delay(1000);
      pixel.show();
    }

    // STATE3 ////////////////////////////////////////////////////////////////////////////////////////////////////

    if (animationState == 3) {
      //      for (uint16_t i = 0; i < pixel.numPixels(); i++) {
      //        pixel.setPixelColor(i, pixel.Color(0, 0, 0));
      //      }
      //      pixel.setBrightness(255);

int speed = 18;
int wipeDelay = 1500;

      //pink
      colorWipe(pixel.Color(57, 2, 0), speed);
      colorWipe(pixel.Color(0, 0, 0), speed);
      colorWipe(pixel.Color(57, 2, 0), speed);
      colorWipe(pixel.Color(0, 0, 0), speed);

       delay(wipeDelay);
      //red
      colorWipe(pixel.Color(57, 0, 20), speed);
      colorWipe(pixel.Color(0, 0, 0), speed);
      colorWipe(pixel.Color(57, 0, 20), speed);
      colorWipe(pixel.Color(0, 0, 0), speed);
        delay(wipeDelay);
    
      //teal
      colorWipe(pixel.Color(0, 170, 226), speed);
      colorWipe(pixel.Color(0, 0, 0), speed);
      colorWipe(pixel.Color(0, 170, 226), speed);
      colorWipe(pixel.Color(0, 0, 0), speed);

      delay(wipeDelay);

        //white
      colorWipe(pixel.Color(0, 0, 0, 200), speed);
      colorWipe(pixel.Color(0, 0, 0), speed);
      colorWipe(pixel.Color(0, 0, 0, 200), speed);
      colorWipe(pixel.Color(0, 0, 0), speed);
 
 delay(wipeDelay);
      pixel.show();
    }

    // STATE4 ////////////////////////////////////////////////////////////////////////////////////////////////////

    if (animationState == 4) {
      pixel.setBrightness(255);
      rainbowCycle(1);
      colorWipe(pixel.Color(0, 0, 0), 30);
      pixel.show();
      delay(2000);

    }
  }
}







// LARSON COLOR FUNCTIONS /////////////////////////////////////////////////////////////////////////////////////////

void larsonScanner1(uint8_t wait) {
  int j;

  for (uint16_t i = 0; i < pixel.numPixels() + 5; i++) {
    // Draw 5 pixels centered on pos.
    //colors is deep purple - appears light purple/pink
    pixel.setPixelColor(pos - 2, 0x771985); // Dark
    pixel.setPixelColor(pos - 1, 0xa010b6); // Medium
    pixel.setPixelColor(pos , 0xd40ff1); // Center pixel is brightest
    pixel.setPixelColor(pos + 1, 0xa010b6); // Medium
    pixel.setPixelColor(pos + 2, 0x771985); // Dark

    pixel.show();
    delay(wait);

    // Rather than being sneaky and erasing just the tail pixel,
    // it's easier to erase it all and draw a new one next time.
    for (j = -2; j <= 2; j++) pixel.setPixelColor(pos + j, 0);

    // Bounce off ends of strip
    pos += dir;
    if (pos < 0) {
      pos = 1;
      dir = -dir;
    } else if (pos >= pixel.numPixels()) {
      pos = pixel.numPixels() - 2;
      dir = -dir;
    }
  }
}

void larsonScanner2(uint8_t wait) {
  int j;

  for (uint16_t i = 0; i < pixel.numPixels() + 5; i++) {
    // Draw 5 pixels centered on pos.
    //colors is dark pink
    pixel.setPixelColor(pos - 2, 0x78003e); // Dark
    pixel.setPixelColor(pos - 1, 0xb3005d); // Medium
    pixel.setPixelColor(pos , 0xff0185); // Center pixel is brightest
    pixel.setPixelColor(pos + 1, 0xb3005d); // Medium
    pixel.setPixelColor(pos + 2, 0x78003e); // Dark

    pixel.show();
    delay(wait);


    // Rather than being sneaky and erasing just the tail pixel,
    // it's easier to erase it all and draw a new one next time.
    for (j = -2; j <= 2; j++) pixel.setPixelColor(pos + j, 0);

    // Bounce off ends of strip
    pos += dir;
    if (pos < 0) {
      pos = 1;
      dir = -dir;
    } else if (pos >= pixel.numPixels()) {
      pos = pixel.numPixels() - 2;
      dir = -dir;
    }
  }
}


// FLASH RANDOM COLOR FUNCTION /////////////////////////////////////////////////////////////////////////////////////////

void flashRandom(int wait, uint8_t howmany) {

  for (uint16_t i = 0; i < howmany; i++) {

    int col = random(FAVCOLORS);
    int rd = myColors[col][0];
    int grn = myColors[col][1];
    int blu = myColors[col][2];

    int k = random(pixel.numPixels());  // get a random pixel from the list

    // now we will 'fade' it in 5 steps
    for (int x = 0; x < 5; x++) {
      int r = rd * (x + 1); r /= 5;
      int g = grn * (x + 1); g /= 5;
      int b = blu * (x + 1); b /= 5;

      pixel.setPixelColor(k, pixel.Color(r, g, b));
      pixel.show();
      delay(wait);
    }
    // & fade out in 5 steps
    for (int y = 5; y >= 0; y--) {
      int r = rd * y; r /= 5;
      int g = grn * y; g /= 5;
      int b = blu * y; b /= 5;

      pixel.setPixelColor(k, pixel.Color(r, g, b));
      pixel.show();
      delay(wait);
    }
  }
}

// COLOR WHEEL FUNCTIONS /////////////////////////////////////////////////////////////////////////////////////////

// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
  for (uint16_t i = 0; i < pixel.numPixels(); i++) {
    pixel.setPixelColor(i, c);
    pixel.show();
    delay(wait);
  }
}

// Input a value 0 to 255 to get a color value. The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if (WheelPos < 85) {
    return pixel.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if (WheelPos < 170) {
    WheelPos -= 85;
    return pixel.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return pixel.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}

void rainbow(uint8_t wait) {
  uint16_t i, j;

  for (j = 0; j < 10; j++) {
    for (i = 0; i < pixel.numPixels(); i++) {
      pixel.setPixelColor(i, Wheel((i + j) & 255));
    }
    pixel.show();
    delay(wait);
  }
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
  uint16_t i, j;

  for (j = 0; j < 256 * 5; j++) { // 5 cycles of all colors on wheel
    for (i = 0; i < pixel.numPixels(); i++) {
      pixel.setPixelColor(i, Wheel(((i * 256 / pixel.numPixels()) + j) & 255));
    }
    pixel.show();
    delay(wait);
  }
}



// FADETEST ////////////////////////////////////////////////////////////////////////#include <Adafruit_NeoPixel.h>

void rgbFadeInAndOutPink(uint8_t wait) {

  uint8_t red [totalLEDs];
  uint8_t green [totalLEDs];
  uint8_t blue [totalLEDs];

  // set colour
  for (int i = 0; i < totalLEDs; i++)
  {
    red [i] = 255;
    green [i] = 0;
    blue [i] = 229;
  }

  for (uint8_t b = 0; b < 255; b++) {
    for (uint8_t i = 0; i < pixel.numPixels(); i++) {
      pixel.setPixelColor(i, red [i] * b / 255, green [i] * b / 255, blue [i] * b / 255);
    }

    pixel.show();
    delay(wait);
  }

  for (uint8_t b = 255; b > 0; b--) {
    for (uint8_t i = 0; i < pixel.numPixels(); i++) {
      pixel.setPixelColor(i, red [i] * b / 255, green [i] * b / 255, blue [i] * b / 255);
    }
    pixel.show();
    delay(wait);
  }
}

void rgbFadeInAndOutRed(uint8_t wait) {

  uint8_t red [totalLEDs];
  uint8_t green [totalLEDs];
  uint8_t blue [totalLEDs];

  // set colour
  for (int i = 0; i < totalLEDs; i++)
  {
    red [i] = 57;
    green [i] = 2;
    blue [i] = 0;
  }

  for (uint8_t b = 0; b < 255; b++) {
    for (uint8_t i = 0; i < pixel.numPixels(); i++) {
      pixel.setPixelColor(i, red [i] * b / 255, green [i] * b / 255, blue [i] * b / 255);
    }

    pixel.show();
    delay(wait);
  }

  for (uint8_t b = 255; b > 0; b--) {
    for (uint8_t i = 0; i < pixel.numPixels(); i++) {
      pixel.setPixelColor(i, red [i] * b / 255, green [i] * b / 255, blue [i] * b / 255);
    }
    pixel.show();
    delay(wait);
  }
}

void rgbFadeInAndOutTeal(uint8_t wait) {

  uint8_t red [totalLEDs];
  uint8_t green [totalLEDs];
  uint8_t blue [totalLEDs];

  // set colour
  for (int i = 0; i < totalLEDs; i++)
  {
    red [i] = 0;
    green [i] = 170;
    blue [i] = 226;
  }

  for (uint8_t b = 0; b < 255; b++) {
    for (uint8_t i = 0; i < pixel.numPixels(); i++) {
      pixel.setPixelColor(i, red [i] * b / 255, green [i] * b / 255, blue [i] * b / 255);
    }

    pixel.show();
    delay(wait);
  }

  for (uint8_t b = 255; b > 0; b--) {
    for (uint8_t i = 0; i < pixel.numPixels(); i++) {
      pixel.setPixelColor(i, red [i] * b / 255, green [i] * b / 255, blue [i] * b / 255);
    }
    pixel.show();
    delay(wait);
  }
}

//yellow 238, 238, 36
void rgbFadeInAndOut4(uint8_t wait) {

  uint8_t red [totalLEDs];
  uint8_t green [totalLEDs];
  uint8_t blue [totalLEDs];

  // set colour
  for (int i = 0; i < totalLEDs; i++)
  {
    red [i] = 255;
    green [i] = 255;
    blue [i] = 255;
  }

  for (uint8_t b = 0; b < 255; b++) {
    for (uint8_t i = 0; i < pixel.numPixels(); i++) {
      pixel.setPixelColor(i, red [i] * b / 255, green [i] * b / 255, blue [i] * b / 255);
    }

    pixel.show();
    delay(wait);
  }

  for (uint8_t b = 255; b > 0; b--) {
    for (uint8_t i = 0; i < pixel.numPixels(); i++) {
      pixel.setPixelColor(i, red [i] * b / 255, green [i] * b / 255, blue [i] * b / 255);
    }
    pixel.show();
    delay(wait);
  }
}

